@@ -1,11 +1,12 @@
 #Copyright 2007-8 Arthur S Goldstein
 #TESTING PHASE
 
+use Parse::Stallion;
 package Parse::Stallion::CSVFH;
+our @ISA=qw(Parse::Stallion);
 use Carp;
 use strict;
 use warnings;
-use Parse::Stallion;
 
 #Copied somewhat from rfc1480
 # see for reference: http://tools.ietf.org/html/rfc4180
@@ -13,6 +14,7 @@ use Parse::Stallion;
 my $row;
 my $name_count;
 my $field_count;
+my $row_number;
 
 my %with_header_csv_rules = (
    file => {
@@ -29,14 +31,22 @@ my %with_header_csv_rules = (
     },
 
    header => {and=>['name', {multiple=>{and=>['COMMA', 'name']}}],
-     evaluation => sub {return $_[0]->{name}},
+     evaluation => sub {$field_count = 0; return $_[0]->{name}},
      on_match => sub {
       #print STDERR "reset fc\n";
       $_[1]->{field_count} = 0}
     },
 
    record => {and=>['field', {multiple=>{and=>['COMMA', 'field']}}],
-     evaluation => sub {return $_[0]->{field}},
+     evaluation => sub {my $field = $_[0]->{field};
+      $row_number++;
+#print STDERR "fc $field_count and nc $name_count\n";
+      if ($name_count != $field_count) {
+         croak ( "Row $row_number has an error in field count got $field_count expected $name_count");
+      }
+     $field_count = 0;
+     return $field;
+},
      on_match => sub {my $object = shift;
        my $hash = shift;
        $hash->{row_number}++;
@@ -49,12 +59,24 @@ my %with_header_csv_rules = (
     },
 
    name => {and=>['field'],
+     evaluation => sub {
+       my $or = shift;
+       my ($value) = values %$or;
+       $name_count++;
+       return $value},
+     unevaluation => sub {croak 'Mismatch on name in heaader'},
      on_match => sub {my $object = shift;
        my $hash = shift; $hash->{name_count}++},
-     on_unmatch => sub {croak 'Mismatch on name in heaader'}
+     on_unmatch => sub {croak 'Mismatch on name in heaader'},
    },
 
    field => {or => ['escaped', 'non_escaped'],
+     evaluation => sub {
+       $field_count++;
+       my $or = shift;
+       my ($value) = values %$or;
+       return $value},
+     unevaluation => sub {croak 'Mismatch on fields in row '.$row},
      on_match => sub {my $object = shift;
        my $hash = shift;
        #print STDERR "increment fc\n";
@@ -106,10 +128,12 @@ my %with_header_csv_rules = (
 
 );
 
-sub new {
-  my $self = shift;
+sub read_in_file_handle {
   my $parameters = shift;
-  my $to_return = new Parse::Stallion({
+  my $file_handle = $parameters->{file_handle};
+  $name_count = 0;
+  $row_number = 0;
+  my $ps = new Parse::Stallion({
     rules_to_set_up_hash=>\%with_header_csv_rules, start_rule=>'file',
      keep_white_space => 1,
      on_start => sub {
@@ -117,15 +141,53 @@ sub new {
        my $hash = $_[1];
        my $fh = $hash->{file_handle} = $object;
        $_[0] = <$fh>;
-      }
+      },
+      parse_function => sub {my $cv = shift;
+#print STDERR "cv is $cv\n";
+#print STDERR "cfh is $file_handle\n";
+        if (defined $cv && $cv ne '') {return $cv};
+        my $next_line = <$file_handle>;
+        if (defined $next_line) {return $next_line};
+        return '';}
     });
   my $ivf = 0;
-  $to_return->set_handle_object({
+  $ps->set_handle_object({
     increasing_value_function => sub {return $ivf++}
   });
-  return $to_return;
+  return $ps->parse_and_evaluate;
 }
 
+#sub xnew {
+#  my $self = shift;
+#  my $parameters = shift;
+#  my $file_handle = $parameters->{file_handle};
+##print STDERR "fh is $file_handle\n";
+#  my $to_return = new Parse::Stallion({
+#    rules_to_set_up_hash=>\%with_header_csv_rules, start_rule=>'file',
+#     keep_white_space => 1,
+#     on_start => sub {
+#       my $object = $_[0];
+#       my $hash = $_[1];
+#       my $fh = $hash->{file_handle} = $object;
+#       $_[0] = <$fh>;
+#      },
+#      parse_function => sub {my $cv = shift;
+##print STDERR "cv is $cv\n";
+##print STDERR "cfh is $file_handle\n";
+#        if (defined $cv && $cv ne '') {return $cv};
+#        my $next_line = <$file_handle>;
+#        if (defined $next_line) {return $next_line};
+#        return '';}
+#    });
+#  my $ivf = 0;
+#  $to_return->set_handle_object({
+#    increasing_value_function => sub {return $ivf++}
+#  });
+#  $name_count = 0;  #should not have this shared
+#  $row_number = 0; #should not have this shared, should not reset here
+#  return $to_return;
+#}
+
 
 1;
 
@@ -141,17 +203,15 @@ Parse::Stallion::CSVFH - Comma Separated Values from file handle
 
   use Parse::Stallion::CSVFH;
 
-  my $csv_stallion = new Parse::Stallion::CSVFH;
-
   my $file_handle;
   open $file_handle, "<", "csv_file";
 
-  my $result = eval {$csv_stallion->
-   parse_and_evaluate({string=>$file_handle})};
+  my $csv_stallion = new Parse::Stallion::CSVFH(file_handle => $file_handle);
+
+  my $result = $csv_stallion->parse_and_evaluate();
+
+  if ($stallion->parse_failed) {#parse failed};
 
-  if ($@) {
-    if ($stallion->parse_failed) {#parse failed};
-  }
   # else $result contains reference to array of arrays
 
 If the file handle refers to a file containing:
